#!/usr/bin/env python

'''
Created on 09.02.2014
@author: Dorjee Tamang
'''

import os, sys
from optparse import OptionParser

from src.Cluster import * #@UnusedWildImport

class Prediction():
            
    def predict(self, options, args):
        '''Returns the prediction result.'''
        
        input_sequences = open(args[0], "r").read()
        
        clus = Cluster(input_sequences, options.unique)
        
        if clus.format == 'fasta':
                peplist = []
                for item in map(lambda s: s.strip(), filter(None, input_sequences.split(">"))):
                    peplist.append(item.splitlines()[1])
        else: 
            peplist = input_sequences.split()
                
        v = []
        for peptide in peplist:
            v.append(self.homologous_sequence(peptide, peplist, options.threshold))
                
        data = []
        epitope_count = []
        cnt = 0
        for lis in self.cluster(v):
            cnt += 1
            count = 0
            cluster = []
            for id, seq in clus.get_dict.items():
                if seq in lis:
                    count += 1
                    cluster.append((count,id,seq))
            data.append(cluster)  
            
        _data = []
        for i, d in enumerate(data):
            dlis_group = []
            for lis in d:
                dlis = list(lis)
                dlis.insert(0, len(d))
                dlis_group.append(dlis)
            _data.append(dlis_group)
        
        _all_data = []
        for i, data in enumerate(_data):
            for d in data:
                _all_data.append(tuple([i+1] + d))
             
        tmpfile_name = self.create_csv(_all_data)
        
        with open(tmpfile_name, "r") as infile:
            print infile.read().replace(',','\t')
    
        os.remove(tmpfile_name)
    
    def cluster(self, v):
        n = self.count_overlap(v)
        if n == 0:
            return v
        else:
            vMerged = []
            for i in range(len(v)):
                if v[i]:
                    v1 = v[i]
                    bAdd = False
                    bDistinct = True
                    for j in range(len(v)):
                        if v[j]:
                            v2 = v[j]
                            if i != j and len(v[j]) > 0:
                                if self.is_overlap(v[i],v[j]):
                                    v1 = list(set(v1 + v2))
                                    v[j] = []
                                    bAdd = True
                                    bDistinct = False
                    
                    if bAdd or bDistinct:
                        if len(v1) > 0:
                            vMerged.append(v1)
            return self.cluster(vMerged)

    def is_overlap(self, i, j):
        return reduce(lambda v1,v2: v1 or v2, map(lambda v: v in i, j))
        
    def count_overlap(self, v):
        count = 0
        for i in v:
            for j in v:
                if i != j:
                    if reduce(lambda v1,v2: v1 or v2, map(lambda v: v in i, j)):
                        count += 1
        return count
     
    def homologous_sequence(self, peptide, peplist, threshold):
        import itertools
        v = []
        v.append(peptide)
        for peptideArray in peplist:
            if peptide != peptideArray:
                peptide1 = peptide
                peptide2 = peptideArray
                if len(peptide2) < len(peptide1):
                    peptide1 = peptideArray
                    peptide2 = peptide
                     
                maxPctIdentity = 0
                
                if peptide1 in peptide2:
                    maxPctIdentity = "%f" %99.9
                else:
                    seqHomology = self.compute_homology(peptide1, peptide2)
                    maxIdentity = seqHomology[0]
                    maxPctIdentity = 100.0 * maxIdentity / len(peptide2)
                    
                if maxPctIdentity >= float(threshold):
                    if not peptideArray in v:
                        v.append(peptideArray)
        return v   
        
    def compute_homology(self, pep, pro):
        proSize = len(pro)
        pepSize = len(pep)
        pepStartIndex = 0
        pepEndIndex = pepSize
    
        maxHomology = 0
        maxOccurence = 0
        
        if proSize >= pepSize:
            vStart = []; vEnd = []; vSeq = []
            
            for i in range(0, proSize-pepSize+1):
                proStartIndex = i;
                proEndIndex = i + pepSize;
                occurence = self.count_residue_match(pep, pepStartIndex, pepEndIndex, pro, proStartIndex, proEndIndex)
                homology = 100 * (float(occurence)/float(pepSize))
                
                if homology >= maxHomology:
                    if homology > maxHomology:
                        del vStart[:]
                        del vEnd[:]
                        del vSeq[:]
                        maxHomology = homology
                        maxOccurence = occurence
                        
                    vStart.append(i+1)
                    vEnd.append(i+pepSize)
                    seq = pro[proStartIndex:pepSize+proStartIndex]
                    vSeq.append(seq)

        return maxOccurence,maxHomology,vStart,vEnd,vSeq,pro
    
    def count_residue_match(self, seq1, startIndex1, endIndex1, seq2, startIndex2, endIndex2):
        nMatch = 0
        len1 = endIndex1 - startIndex1
        len2 = endIndex2 - startIndex2
        if len1 == len2:
            for i in range(len1):
                if seq1[i + startIndex1] == seq2[i + startIndex2]:
                    nMatch += 1
        return nMatch
    
    def create_csv(self, data):
        import csv 
        import tempfile
        
        cluster_tmpdir = './output'
        
        # create a temporary file inside the tmp/ directory
        tmpfile = tempfile.NamedTemporaryFile(prefix="csv_", dir=cluster_tmpdir, delete=False)
        
        with open(tmpfile.name, 'wb') as result:
            writer = csv.writer(result, delimiter=',')
            writer.writerow(('cluster_number','num_epitopes_in_cluster','epitope_num','epitope_name','epitope_seq'))
            for score in data:
                writer.writerow(score)
        tmpfile.close()
        return tmpfile.name

                
    def commandline_help(self):
        print """
* Available values:
-------------------
threshold: 0-100
unique: on, off
 
1) To use the tool with default parameters (threshold=80, unique=on):
---------------------------------------------------------------------
python predict_cluster.py [input-file]
Example: python predict_cluster.py example/test.txt
 
2) To use the tool with specific parameters:
--------------------------------------------
python predict_cluster.py [options] [input-file]
Example: python predict_cluster.py --threshold=75 --unique=off example/test.txt
 
You can also use help option (-h or --help) for more information:
-----------------------------------------------------------------
python predict_bcell.py --help
"""

    def main(self):
        
        parser = OptionParser(usage="usage: %prog [options] input", version="%prog 1.0")
        
        parser.add_option('-t', '--threshold',
                          default='80',
                          help="Specify a threshold value for the clustering (default=80).")
        
        parser.add_option('-u', '--unique', 
                          type='choice', 
                          choices=['on','off'],
                          default='on',
                          help="Show/hide redundant epitope sequence(s) (default=on).")
        
        (options, args) = parser.parse_args()
        if len(args) == 0: 
            parser.error("Input file is not specified.")
        
        if os.path.isfile(args[0]) == False:
            self.commandline_help()
            sys.exit(0)
            
        self.predict(options, args)

if __name__ == '__main__':
    Prediction().main()

